@@ -59,7 +59,7 @@ static const char *serpent_test (void);
 #define byte_swap_32(x) \
   (0 \
    | (((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) \
-   | (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24));
+   | (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))
 
 /* These are the S-Boxes of Serpent.  They are copied from Serpents
    reference implementation (the optimized one, contained in
@@ -907,7 +907,7 @@ serpent_test (void)
       },
       {
 	0
-      }
+      },
     };
 
   for (i = 0; test_data[i].key_length; i++)
@@ -953,26 +953,26 @@ serpent_test (void)
 static const char *cipher_spec_serpent128_aliases[] =
   {
     "SERPENT",
-    NULL,
+    NULL
   };
 
 gcry_cipher_spec_t _gcry_cipher_spec_serpent128 =
   {
     "SERPENT128", cipher_spec_serpent128_aliases, NULL, 16, 128,
     sizeof (serpent_context_t),
-    serpent_setkey, serpent_encrypt, serpent_decrypt, NULL, NULL
+    serpent_setkey, serpent_encrypt, serpent_decrypt
   };
 
 gcry_cipher_spec_t _gcry_cipher_spec_serpent192 =
   {
     "SERPENT192", NULL, NULL, 16, 192,
     sizeof (serpent_context_t),
-    serpent_setkey, serpent_encrypt, serpent_decrypt, NULL, NULL
+    serpent_setkey, serpent_encrypt, serpent_decrypt
   };
 
 gcry_cipher_spec_t _gcry_cipher_spec_serpent256 =
   {
     "SERPENT256", NULL, NULL, 16, 256,
     sizeof (serpent_context_t),
-    serpent_setkey, serpent_encrypt, serpent_decrypt, NULL, NULL
+    serpent_setkey, serpent_encrypt, serpent_decrypt
   };
