@@ -63,6 +63,11 @@ static int no_fips_mode_required;
 /* Flag to indicate that we are in the enforced FIPS mode.  */
 static int enforced_fips_mode;
 
+/* If this flag is set, the application may no longer assume that the
+   process is running in FIPS mode.  This flag is protected by the
+   FSM_LOCK.  */
+static int inactive_fips_mode;
+
 /* This is the lock we use to protect the FSM.  */
 static ath_mutex_t fsm_lock = ATH_MUTEX_INITIALIZER;
 
@@ -131,9 +136,9 @@ _gcry_initialize_fips_mode (int force)
 
   /* Checking based on /proc file properties.  */
   {
+    static const char procfname[] = "/proc/sys/crypto/fips_enabled";
     FILE *fp;
     int saved_errno;
-    static const char const procfname[] = "/proc/sys/crypto/fips_enabled";
 
     fp = fopen (procfname, "r");
     if (fp)
@@ -257,9 +262,9 @@ unlock_fsm (void)
 int
 _gcry_fips_mode (void)
 {
-  /* No locking is required becuase we have the requirement that this
-     variable is only intialized once with no other threads
-     exiisting.  */
+  /* No locking is required because we have the requirement that this
+     variable is only initialized once with no other threads
+     existing.  */
   return !no_fips_mode_required;
 }
 
@@ -272,6 +277,54 @@ _gcry_enforced_fips_mode (void)
 }
 
 
+/* If we do not want to enforce the fips mode, we can set a flag so
+   that the application may check whether it is still in fips mode.
+   TEXT will be printed as part of a syslog message.  This function
+   may only be be called if in fips mode. */
+void
+_gcry_inactivate_fips_mode (const char *text)
+{
+  gcry_assert (_gcry_fips_mode ());
+
+  if (_gcry_enforced_fips_mode () )
+    {
+      /* Get us into the error state. */
+      fips_signal_error (text);
+      return;
+    }
+
+  lock_fsm ();
+  if (!inactive_fips_mode)
+    {
+      inactive_fips_mode = 1;
+      unlock_fsm ();
+#ifdef HAVE_SYSLOG
+      syslog (LOG_USER|LOG_WARNING, "Libgcrypt warning: "
+              "%s - FIPS mode inactivated", text);
+#endif /*HAVE_SYSLOG*/
+    }
+  else
+    unlock_fsm ();
+}
+
+
+/* Return the FIPS mode inactive flag.  If it is true the FIPS mode is
+   not anymore active.  */
+int
+_gcry_is_fips_mode_inactive (void)
+{
+  int flag;
+
+  if (!_gcry_fips_mode ())
+    return 0;
+  lock_fsm ();
+  flag = inactive_fips_mode;
+  unlock_fsm ();
+  return flag;
+}
+
+
+
 static const char *
 state2str (enum module_states state)
 {
@@ -730,6 +783,7 @@ fips_new_state (enum module_states new_state)
       
     case STATE_ERROR:
       if (new_state == STATE_SHUTDOWN
+          || new_state == STATE_ERROR
           || new_state == STATE_FATALERROR
           || new_state == STATE_SELFTEST)
         ok = 1;
